/********************************************************************************/
/*										*/
/*			IWG EK Index Parsing Utilities				*/
/*			     Written by Ken Goldman				*/
/*		       IBM Thomas J. Watson Research Center			*/
/*	      $Id: ekutils.h 682 2016-07-15 18:49:19Z kgoldman $		*/
/*										*/
/* (c) Copyright IBM Corporation 2016.						*/
/*										*/
/* All rights reserved.								*/
/* 										*/
/* Redistribution and use in source and binary forms, with or without		*/
/* modification, are permitted provided that the following conditions are	*/
/* met:										*/
/* 										*/
/* Redistributions of source code must retain the above copyright notice,	*/
/* this list of conditions and the following disclaimer.			*/
/* 										*/
/* Redistributions in binary form must reproduce the above copyright		*/
/* notice, this list of conditions and the following disclaimer in the		*/
/* documentation and/or other materials provided with the distribution.		*/
/* 										*/
/* Neither the names of the IBM Corporation nor the names of its		*/
/* contributors may be used to endorse or promote products derived from		*/
/* this software without specific prior written permission.			*/
/* 										*/
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS		*/
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT		*/
/* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR	*/
/* A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT		*/
/* HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,	*/
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT		*/
/* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,	*/
/* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY	*/
/* THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT		*/
/* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE	*/
/* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.		*/
/********************************************************************************/

#ifndef EKUTILS_H
#define EKUTILS_H

#include <openssl/x509.h>
#include <openssl/bn.h>

#include <tss2/tss.h>

/* legacy TCG IWG NV indexes */

#define EK_CERT_RSA_INDEX 	0x01c00002
#define EK_NONCE_RSA_INDEX 	0x01c00003 
#define EK_TEMPLATE_RSA_INDEX 	0x01c00004

#define EK_CERT_EC_INDEX 	0x01c0000a
#define EK_NONCE_EC_INDEX 	0x01c0000b
#define EK_TEMPLATE_EC_INDEX 	0x01c0000c

#define MAX_ROOTS		100	/* 100 should be more than enough */

TPM_RC getIndexSize(TSS_CONTEXT *tssContext,
		    uint16_t *dataSize,
		    TPMI_RH_NV_INDEX nvIndex);
TPM_RC getIndexData(TSS_CONTEXT *tssContext,
		    unsigned char **buffer,
		    TPMI_RH_NV_INDEX nvIndex,
		    uint16_t dataSize);
TPM_RC getIndexContents(TSS_CONTEXT *tssContext,
			unsigned char **buffer,
			uint16_t *bufferSize,
			TPMI_RH_NV_INDEX nvIndex);
void getRsaTemplate(TPMT_PUBLIC *tpmtPublic);
void getEccTemplate(TPMT_PUBLIC *tpmtPublic);
TPM_RC getIndexX509Certificate(TSS_CONTEXT *tssContext,
			       X509 **certificate,
			       TPMI_RH_NV_INDEX nvIndex);
TPM_RC getRootCertificateFilenames(char *rootFilename[],
				   unsigned int *rootFileCount,
				   const char *listFilename);
TPM_RC getCaStore(X509_STORE **caStore,
		  X509 *caCert[],
		  const char *rootFilename[],
		  unsigned int rootFileCount);
TPM_RC processEKNonce(TSS_CONTEXT *tssContext,
		      unsigned char **nonce,
		      uint16_t *nonceSize,
		      TPMI_RH_NV_INDEX ekNonceIndex,
		      int print);
TPM_RC processEKTemplate(TSS_CONTEXT *tssContext,
			 TPMT_PUBLIC *tpmtPublic,
			 TPMI_RH_NV_INDEX ekTemplateIndex,
			 int print);
TPM_RC processEKCertificate(TSS_CONTEXT *tssContext,
			    X509 **ekCertificate,
			    uint8_t **modulusBin,
			    int *modulusBytes,
			    TPMI_RH_NV_INDEX ekCertIndex,
			    int print);
TPM_RC convertCertificatePubKey(uint8_t **modulusBin,	/* freed by caller */
				int *modulusBytes,
				X509 *ekCertificate,
				TPMI_RH_NV_INDEX ekCertIndex,
				int print);
TPM_RC processRoot(TSS_CONTEXT *tssContext,
		   TPMI_RH_NV_INDEX ekCertIndex,
		   const char *rootFilename[],
		   unsigned int rootFileCount,
		   int print);
TPM_RC processCreatePrimary(TSS_CONTEXT *tssContext,
			    TPM_HANDLE *keyHandle,
			    TPMI_RH_NV_INDEX ekCertIndex,
			    unsigned char *nonce,
			    uint16_t nonceSize,
			    TPMT_PUBLIC *tpmtPublicIn,
			    TPMT_PUBLIC *tpmtPublicOut,
			    unsigned int noFlush,
			    int print);
TPM_RC processValidatePrimary(uint8_t *publicKeyBin,
			      int publicKeyBytes,
			      TPMT_PUBLIC *tpmtPublic,
			      TPMI_RH_NV_INDEX ekCertIndex,
			      int print);
TPM_RC processPrimary(TSS_CONTEXT *tssContext,
		      TPMI_RH_NV_INDEX ekCertIndex,
		      TPMI_RH_NV_INDEX ekNonceIndex, 
		      TPMI_RH_NV_INDEX ekTemplateIndex,
		      unsigned int noFlush,
		      int print);
#endif
