REM #############################################################################
REM										#
REM			TPM2 regression test					#
REM			     Written by Ken Goldman				#
REM		       IBM Thomas J. Watson Research Center			#
REM		$Id: initkeys.bat 593 2016-05-18 15:04:15Z kgoldman $	#
REM										#
REM (c) Copyright IBM Corporation 2015						#
REM 										#
REM All rights reserved.							#
REM 										#
REM Redistribution and use in source and binary forms, with or without		#
REM modification, are permitted provided that the following conditions are	#
REM met:									#
REM 										#
REM Redistributions of source code must retain the above copyright notice,	#
REM this list of conditions and the following disclaimer.			#
REM 										#
REM Redistributions in binary form must reproduce the above copyright		#
REM notice, this list of conditions and the following disclaimer in the		#
REM documentation and/or other materials provided with the distribution.	#
REM 										#
REM Neither the names of the IBM Corporation nor the names of its		#
REM contributors may be used to endorse or promote products derived from	#
REM this software without specific prior written permission.			#
REM 										#
REM THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS		#
REM "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT		#
REM LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR	#
REM A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT	#
REM HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,	#
REM SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT		#
REM LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,	#
REM DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY	#
REM THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT		#
REM (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE	#
REM OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.	#
REM										#
REM #############################################################################

setlocal enableDelayedExpansion
 
echo | set /p="1234567890123456" > msg.bin
touch zero.bin

REM try to undefine any NV index left over from a previous test.  Do not check for errors.
%TPM_EXE_PATH%nvundefinespace -hi p -ha 01000000 > run.out
%TPM_EXE_PATH%nvundefinespace -hi p -ha 01000000 -pwdp ppp > run.out
%TPM_EXE_PATH%nvundefinespace -hi p -ha 01000001 > run.out
REM same for persistent objects
%TPM_EXE_PATH%evictcontrol -ho 81800000 -hp 81800000 -hi p > run.out

echo ""
echo "Initialize Regression Test Keys"
echo ""

echo "Create a platform primary storage key"
%TPM_EXE_PATH%createprimary -hi p -pwdk pps > run.out
IF !ERRORLEVEL! NEQ 0 (
  exit /B 1
)

echo "Create a storage key under the primary key"
%TPM_EXE_PATH%create -hp 80000000 -st -kt f -kt p -opr storepriv.bin -opu storepub.bin -pwdp pps -pwdk sto > run.out
IF !ERRORLEVEL! NEQ 0 (
  exit /B 1
)

echo "Create an unrestricted RSA signing key under the primary key"
%TPM_EXE_PATH%create -hp 80000000 -si -kt f -kt p -opr signpriv.bin -opu signpub.bin -pwdp pps -pwdk sig > run.out
IF !ERRORLEVEL! NEQ 0 (
  exit /B 1
)

echo "Create an unrestricted ECC signing key under the primary key"
%TPM_EXE_PATH%create -hp 80000000 -ecc nistp256 -si -kt f -kt p -opr signeccpriv.bin -opu signeccpub.bin -pwdp pps -pwdk sig > run.out
IF !ERRORLEVEL! NEQ 0 (
  exit /B 1
)

echo "Create a restricted signing key under the primary key"
%TPM_EXE_PATH%create -hp 80000000 -sir -kt f -kt p -opr signrpriv.bin -opu signrpub.bin -pwdp pps -pwdk sig > run.out
IF !ERRORLEVEL! NEQ 0 (
  exit /B 1
)

echo "Create an RSA decryption key under the primary key"
%TPM_EXE_PATH%create -hp 80000000 -den -kt f -kt p -opr derpriv.bin -opu derpub.bin -pwdp pps -pwdk dec > run.out
IF !ERRORLEVEL! NEQ 0 (
  exit /B 1
)

echo "Create a symmetric cipher key under the primary key"
%TPM_EXE_PATH%create -hp 80000000 -des -kt f -kt p -opr despriv.bin -opu despub.bin -pwdp pps -pwdk aes > run.out
IF !ERRORLEVEL! NEQ 0 (
  exit /B 1
)

for %%H in (sha1 sha256 sha384) do (
    echo "Create a %%H keyed hash key under the primary key"
%TPM_EXE_PATH%create -hp 80000000 -kh -kt f -kt p -opr khpriv%%H.bin -opu khpub%%H.bin -pwdp pps -pwdk khk -halg %%H
    IF !ERRORLEVEL! NEQ 0 (
       exit /B 1
       )
)

exit /B 0
